#include <cmath>
#include "grid.h"
#include "primitivebuilder.h"

using namespace std;

namespace Msp {
namespace GL {

GridBuilder::GridBuilder(float w, float h, unsigned u, unsigned v):
	GridBuilder(Vector3(-w/2, -h/2, 0), Vector3(w, 0, 0), Vector3(0, h, 0), u, v)
{ }

GridBuilder::GridBuilder(const Vector3 &o, const Vector3 &s, unsigned u, unsigned v):
	GridBuilder(o, get_side(s, 0), get_side(s, 1), u, v)
{ }

GridBuilder::GridBuilder(const Vector3 &o, const Vector3 &s1, const Vector3 &s2, unsigned u, unsigned v):
	origin(o),
	side1(s1),
	side2(s2),
	u_div(max(u, 1U)),
	v_div(max(v, 1U))
{
	norm = normalize(cross(side1, side2));
	binorm = normalize(cross(norm, side1));
}

Vector3 GridBuilder::get_side(const Vector3 &size, unsigned index)
{
	if(!size.x)
		return Vector3(0, (index==0 ? size.y : 0), (index==1 ? size.z : 0));
	else if(!size.y)
		return Vector3((index==0 ? size.x : 0), 0, (index==1 ? size.z : 0));
	else
		return Vector3((index==0 ? size.x : 0), (index==1 ? size.z : 0), 0);
}

void GridBuilder::build(PrimitiveBuilder &builder) const
{
	float l1_sq = dot(side1, side1);
	float l2 = dot(side2, binorm);
	float u_scale = 1/l1_sq;
	float v_scale = 1/l2;
	adjust_texture_scale(u_scale, v_scale, sqrt(l1_sq), l2);

	builder.normal(norm.x, norm.y, norm.z);
	if(generate_tan)
		builder.tangent(side1);

	for(unsigned j=0; j<=v_div; ++j)
	{
		Vector3 row = side2*(j*1.0f/v_div);
		float v = dot(row, binorm)*v_scale;
		for(unsigned i=0; i<=u_div; ++i)
		{
			Vector3 p = row+side1*(i*1.0f/u_div);
			float u = dot(p, side1)*u_scale;
			builder.texcoord(u, v);
			builder.vertex(origin+p);
		}
	}

	for(unsigned j=0; j<v_div; ++j)
	{
		builder.begin(TRIANGLE_STRIP);
		for(unsigned i=0; i<=u_div; ++i)
		{
			builder.element((j+1)*(u_div+1)+i);
			builder.element(j*(u_div+1)+i);
		}
		builder.end();
	}
}

} // namespace GL
} // namespace Msp
